//---------------------------------------------------------------------------

#ifndef SerialHandlerH
#define SerialHandlerH
//---------------------------------------------------------------------------


// forward declarations; don't need to include the headers
class TDevice;
class TDataHandler;

class Classes::TStrings;

// CPort class is used from delphi; in delphi, each program unit is
//   handled as a separate namespace
namespace Cport
{
	class TComPort;
}

// log row forware declare
struct TLogRow;

//---------------------------------------------------------------------------
class TSerialHandler
{

private:
	// the 'device' instance which handles incoming frames
	TDevice* FDevice;
	TDataHandler* FDataHandler;
    // com port used to send/receive data
    Cport::TComPort* FCOMPort;

	// incoming data, first handling point
	void __fastcall DataAvailableFromComPort(TObject* Sender, int LCount);

public:
	// constructor and destructor, create/destroy port and buffers
	TSerialHandler(TDataHandler& ADataHandler);
	~TSerialHandler();

	// when last write to port occured; and when after that we received stuff
	__int64 FLastSendTime;
	double FLastLatency;

	// return pointer to list WHICH IS OWNED BY THIS CLASS
    static TStrings* GetPortList();

    // ** com port control functions
    bool IsPortOpen();

	// create port and set defaults (moved here because reopen)
	void CreateSerialPort();

	// open/close serial port
	bool OpenCOMPort(int aComPort);
	void CloseCOMPort();
    void SetBaudRate(int BaudRate);
    void SetDTRRTS(bool DTR, bool RTS);

    // write/read port
    void Write(void* APtrToData, int ALength);
	int Read(void* APtrToData, int ALength);

	// set true to enable logging
	void SetLogEnabled(bool AIsEnabled);
	bool FLoggingEnabled;
	// reset start timestamp
	void ResetLogStartTimes();

	// log start timestamp, set when log is enabled
	_LARGE_INTEGER FLoggingStarted;
	// pointer to first log entry, linked list
	TLogRow* FReadLogEntryList;
	TLogRow* FWriteLogEntryList;
};

struct TLogRow
{
public:
	// message timestamp (reduce this from timer when "log enabled" to acquire nice timestamps)
	__int64 FTimestamp;

	// byte count
	int FByteCount;
	unsigned char FMessage[32];
	// pointer to long (separately allocated) message if necessary
	unsigned char* FLongMessagePtr;

	// 2-way linked list, pointers
	TLogRow* FPtrToPrev;
	TLogRow* FPtrToNext;

	// init new log row with possible previous pointer and timestamp
	TLogRow(TLogRow* ALinkToPrevious=NULL)
	{
		memset(this, 0, sizeof(*this));
		// timestamp of the message
		_LARGE_INTEGER LLargeTimestamp;
		QueryPerformanceCounter(&LLargeTimestamp);
		FTimestamp = LLargeTimestamp.QuadPart;

		FPtrToPrev = ALinkToPrevious;
	}

	~TLogRow()
	{
		// free reserved memory from long message pointer. short message pointer does not require free
		if (FLongMessagePtr)
		{
			delete FLongMessagePtr;
			FLongMessagePtr=NULL;
		}
    }

	// get message pointer, either in struct or memory stored
	unsigned char* GetMessagePtr()
	{
		if (FLongMessagePtr == NULL)
		{
			return FMessage;
		}
		return FLongMessagePtr;
	}

	// set binary message, convert in to hex format
	void SetHexMessage(char* AMessage, int AByteCount)
	{
		const char HexChars[]="0123456789ABCDEF";
		char* LHex = new char[3*AByteCount+1];
        // prepare string with spaces, but give ending null byte to end the string
		memset(LHex, 32, 3*AByteCount+1);
		LHex[3*AByteCount] = 0;

		for(int i=0;i < AByteCount; i++)
		{
			char LByte = AMessage[i];
			LHex[3*i  ] = HexChars[(LByte & 0xf0) >> 4];
			LHex[3*i+1] = HexChars[ LByte & 0x0f ];
		}

		FLongMessagePtr = LHex;
		FByteCount = 3*AByteCount;
	}

	// set ascii message
	void SetMessage(char* AMessage, int AByteCount)
	{
		if (AByteCount <= 32)
		{
			memcpy(FMessage, AMessage, AByteCount);
			FByteCount = AByteCount;
		}
		else
		{

			FLongMessagePtr = (unsigned char*) new char[AByteCount];
			FByteCount = AByteCount;
			memcpy(FLongMessagePtr, AMessage, AByteCount);
        }
	}
};

#endif