﻿//---------------------------------------------------------------------------
// UnitFormStartMenu.cpp
//
// Noptel/FN Expert, NOS Pro
// C. Noptel Oy, Oulu Finland, 2011-2017
//
// Application Start up menu with Applications Settings and main buttons.
// Includes non-documented diagnostic log to view serial connections (scanning).
//
// Reference: Noptel Doc# N41839: Noptel Expert Application - User's Guide 2.1
//            Chapter 2.1 Start up
//
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UnitFormStartMenu.h"

#include "UnitFormSelectSetup.h"

#include "UnitFormPrepareDownload.h"

#include "UnitFormNetworkControl.h"

#include "Device.h"

#include "DeviceManager.h"

#include "UnitFormOpenSession.h"

#include "UnitFormApplicationSettings.h"

#include "Globals.h"

#include "ShotData.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormStartMenu *FormStartMenu;

//---------------------------------------------------------------------------
__fastcall TFormStartMenu::TFormStartMenu(TComponent* Owner)
	: TForm(Owner)
{
	AddToLog(CCreatedByVersion);
	AddToLog(FormatDateTime("yyyy-mm-dd tt",Now()));

	if( IsAdvancedSettingsEnabled() == true)
	{
		ButtonAppSettings->Glyph = ImageAdvancedSettings->Picture->Bitmap;
		//ImageAdvancedSettings->Visible = true;
	}

	// display custom logo 2 (smaller) instead of "Noptel Expert App" logo, if found
	if (FileExists("bitmaps\\logo2.bmp"))
	{
		try
		{
			ImageLogo2->Picture->Bitmap->LoadFromFile("bitmaps\\logo2.bmp");
			ImageLogo2->Visible = true;
			ImageExpertApplication->Visible = false;
		}
		catch (Exception& e)
		{
			// TODO
		}
	}
	// use custom logo (larger) from start splash, if exist
	else if (FileExists("bitmaps\\logo.bmp"))
	{
		try
		{
			ImageLogo2->Picture->Bitmap->LoadFromFile("bitmaps\\logo.bmp");
			// rearrange
			ImageLogo2->Left = 0;
			ImageLogo2->Top = 0;
			ImageLogo2->Height = 200;
			ImageLogo2->Width = 250;
			ImageLogo2->Visible = true;
			ImageExpertApplication->Visible = false;
		}
		catch (Exception& e)
		{
			// TODO
		}
	}
	if (FileExists("bitmaps\\logo3.bmp"))
	{
		try
		{
			ImageLogo3->Picture->Bitmap->LoadFromFile("bitmaps\\logo3.bmp");
			ImageLogo3->Visible = true;
		}
		catch (Exception& e)
		{
			// TODO
		}
	}

}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TFormStartMenu::BitBtnCloseClick(TObject *Sender)
{
	// Check if settings need to be saved
	ApplyAndSaveSettings();

	Close();
}
//---------------------------------------------------------------------------

void __fastcall TFormStartMenu::FormClose(TObject *Sender, TCloseAction &Action)
{
	Application->Terminate();    
}
//---------------------------------------------------------------------------

void __fastcall TFormStartMenu::BitBtnClassicTrainingClick(TObject *Sender)
{
	GSelectedMode = KClassicTraining;

	FormSelectSetup->CheckBoxModeDynamic->Checked = false;

	FormSelectSetup->Show();

	Hide();
}
//---------------------------------------------------------------------------

void __fastcall TFormStartMenu::BitBtnOpenSessionClick(TObject *Sender)
{
	if (FormNetworkControl->FServerSocket->Active)
	{

		FormNetworkControl->FServerSocket->Active = false;
		FormNetworkControl->CheckBoxServerActive->Checked = false;
		LabelWaitingForConnections->Hide();
	}

	GSelectedMode = KOpenSession;

	FormOpenSession->Show();

	Hide();
}
//---------------------------------------------------------------------------

void __fastcall TFormStartMenu::BitBtnDownloadSessionClick(TObject *Sender)
{
	GSelectedMode = KDownload;

	FormPrepareDownload->CheckBoxClearShotsAfterDownload->Checked = false;
	if (GAppSettings->GetNode(CAppSettingsOptionsNode))
		if (GAppSettings->GetNode(CAppSettingsOptionsNode)->GetNode(CAppSettingsOptionsClearShotsAfterDownload))
			if (GAppSettings->GetNode(CAppSettingsOptionsNode)->GetNodeText(CAppSettingsOptionsClearShotsAfterDownload).LowerCase() == "true")
				FormPrepareDownload->CheckBoxClearShotsAfterDownload->Checked = true;

	FormPrepareDownload->Show();

	Hide();

	// start time
	GSession->FDateTime = FormatDateTime("ddddd t", Now());

}
//---------------------------------------------------------------------------


void __fastcall TFormStartMenu::BitBtnDynamicTrainingClick(TObject *Sender)
{
	GSelectedMode = KDynamicTraining;

	FormSelectSetup->CheckBoxModeDynamic->Checked = true;

	FormSelectSetup->Show();

	Hide();
	
}
//---------------------------------------------------------------------------


void __fastcall TFormStartMenu::TimerUpdateDeviceCountsTimer(TObject *Sender)
{
	int LDevices = GetDeviceManager()->GetFoundCount();

	int LScanning = GetDeviceManager()->GetScanningCount();

	UnicodeString LCount;
	LCount = LDevices;

	if (LScanning)
	{
		LCount.cat_printf(L" (%d)", LScanning);
	}

	LabelDeviceCount->Caption = LCount;
	FormSelectSetup->LabelDeviceCount->Caption = LCount;
	FormPrepareDownload->LabelDeviceCount->Caption = LCount;

	// if we are set to operate as network monitor, disable set/start buttons
	bool LMonitor = (GetNetworkMode() == CAppSettingsNetworkNodeModeMonitor);

	// if we are scanning, disable UI buttons...
	if (LScanning || LCount == 0 || LMonitor)
	{
		FormSelectSetup->DisableUI();
	}
	else
	{
		// enable them when no programming is going
		if (FormSelectSetup->FSetOrStartGoing == 0)
		{
			FormSelectSetup->EnableUI();
		}
	}

	if (LScanning) {
		BitBtnDownloadSession->Enabled = false;
		BitBtnOpenSession->Enabled = false;
	}
	else
	{
		BitBtnDownloadSession->Enabled = true;
		BitBtnOpenSession->Enabled = true;
	}

	// update device count/type image according
	int LTagToBe = 0;
	if (LCount == 0)
	{
		if (ImageDeviceCount->Picture != ImageDevicesNone->Picture)
		int LTagToBe = 0;
	}
	else
	{
		if (!GDeviceManager->GetDevice(0)->FIsBluetooth)
		{
            LTagToBe = 1;
		}
		else
		{
			LTagToBe = 2;
		}
	}

	// if need to change the image...
	if (LTagToBe != ImageDeviceCount->Tag)
	{
		if (LTagToBe == 0)
			ImageDeviceCount->Picture = ImageDevicesNone->Picture;
		if (LTagToBe == 1)
			ImageDeviceCount->Picture = ImageDevicesWired->Picture;
		if (LTagToBe == 2)
			ImageDeviceCount->Picture = ImageDevicesBluetooth->Picture;

		FormSelectSetup->ImageDeviceCountIcon->Picture = ImageDeviceCount->Picture;
		FormPrepareDownload->ImageDeviceCountIcon->Picture = ImageDeviceCount->Picture;
		ImageDeviceCount->Tag = LTagToBe;
	}

	// as monitor, show number of connected shooters
	if (LMonitor)
	{
		LabelDeviceCount->Caption = FormNetworkControl->FClientShootersList->Count;
		// hide start menu when incoming shooters
		if (FormNetworkControl->FClientShootersList->Count > 0 && Visible)
		{
			Hide();
		}
	}

	UnicodeString LScanningStr = "";

	for (int i = 0; i < GDeviceManager->GetDeviceCount(); i++)
	{
		if (GDeviceManager->GetDevice(i)->FAsyncInfo.FScanGoing &&
			!GDeviceManager->GetDevice(i)->FAsyncInfo.FScanDone &&
			GDeviceManager->GetDevice(i)->FCOMPort > 0)
		{
			LScanningStr.cat_printf(L" COM%d", GDeviceManager->GetDevice(i)->FCOMPort);
		}
	}

	if (LScanning == 0) LScanningStr = "";

	LabelPorts->Caption = LScanningStr;

}
//---------------------------------------------------------------------------

void __fastcall TFormStartMenu::FormShow(TObject *Sender)
{
	// translations
	WGetTranslation("Waiting For Connections");

	UpdateButtons();

	ImageDeviceCount->Hint = WGetTranslation("Click to reconnect device(s)");

	WGetTranslation("Waiting For Connections");
}
//---------------------------------------------------------------------------

void TFormStartMenu::UpdateButtons()
{
	bool LWhatShow = (GetNetworkMode() == CAppSettingsNetworkNodeModeMonitor);

	//BitBtnClassicTraining->Visible = !LWhatShow;
	BitBtnDynamicTraining->Visible = !LWhatShow;
	BitBtnDownloadSession->Visible = !LWhatShow;

	BitBtnMonitorTraining->Visible = LWhatShow;

	if (!LWhatShow)
	{
		LabelWaitingForConnections->Visible = false;

		if (FormNetworkControl->FServerSocket->Active)
		{
			FormNetworkControl->FServerSocket->Active == false;
		}
	}
}

void __fastcall TFormStartMenu::BitBtnMonitorTrainingClick(TObject *Sender)
{
	// just enable
	if (FormNetworkControl->FServerSocket->Active == false)
	{
		if (GAppSettings->GetNode(CAppSettingsNetworkNode) )
		{
			FormNetworkControl->FServerSocket->Port = GAppSettings->GetNode(CAppSettingsNetworkNode)->GetAttributeValueDef(CAppSettingsNetworkNodeMonitorPort,14351);
		}

		FormNetworkControl->FServerSocket->Active = true;
		FormNetworkControl->CheckBoxServerActive->Checked = true;
		LabelWaitingForConnections->Caption = WGetTranslation("Waiting For Connections");
		LabelWaitingForConnections->Visible = true;

		PlaceOnCenter(LabelWaitingForConnections);

		GSelectedMode = KNetworkMonitor;
	}
	else
	{
		FormNetworkControl->ServerClearClients();
		FormNetworkControl->FServerSocket->Active = false;
		FormNetworkControl->CheckBoxServerActive->Checked = false;

		//BitBtnMonitorTraining->SendToBack();
		//BitBtnClassicTraining->Visible = true;
		//BitBtnDynamicTraining->Visible = true;
		//BitBtnDownloadSession->Visible = true;

		LabelWaitingForConnections->Visible = false;

		GSelectedMode = KNoneMode;
	}
}
//---------------------------------------------------------------------------

void __fastcall TFormStartMenu::ButtonAppSettingsClick(TObject *Sender)
{
	// if monitoring, stop it before entering app settings
	if (GSelectedMode == KNetworkMonitor && FormNetworkControl->FServerSocket->Active == true)
	{
		BitBtnMonitorTrainingClick(NULL);
	}

	if (FormApplicationSettings->ShowModal() == mrOk)
	{
		// update
		UpdateButtons();
	}
}
//---------------------------------------------------------------------------

void __fastcall TFormStartMenu::ImageDeviceCountClick(TObject *Sender)
{
	ImageDeviceCount->Enabled = false;

	AddBusyCount();

	// updated by scan timer
	BitBtnDownloadSession->Enabled = false;
	BitBtnOpenSession->Enabled = false;
	BitBtnDownloadSession->Update();
	BitBtnOpenSession->Update();

	// enable bluetooth scan when this is clicked
	GCheckForBluetooth = true;

	// close ports so if there are "dead" BT ports they are rescanned
	while (GDeviceManager->GetDeviceCount() > 0)
	{
		TDevice* LDevice = GDeviceManager->GetDevice(0);
		try {
			// TODO CleanAndDelete device function (this is used 2/3 times)
			if (LDevice->FShooter)
			{
				// Don't delete shooters, they are deleted before programming
				GSession->FShooters->Extract(LDevice->FShooter);
				delete LDevice->FShooter;
				LDevice->FShooter = NULL;
			}
			LDevice->CloseComPort();
			delete LDevice;
		}
		catch (...)
		{
			delete LDevice;
		}
		GDeviceManager->FDevicesList->Extract(LDevice);
	}

	// just for case
	Sleep(4000);

	// create, connect, no download
	GDeviceManager->CheckForDevices(true, true, false);

	// disabled scanning for bluetooth device until further
	//GCheckForBluetooth = false;

	ImageDeviceCount->Enabled = true;

    DecBusyCount();
}
//---------------------------------------------------------------------------

void __fastcall TFormStartMenu::LabelDeviceCountClick(TObject *Sender)
{

	// view & hide diagnostics memo log
	MemoLog->Visible = !MemoLog->Visible;

	// update label hint
	if (MemoLog->Visible)
		LabelDeviceCount->Hint="Hide Diagnostics";
	else
		LabelDeviceCount->Hint="View Diagnostics";

	// additional feature for communication diagnostics
	// to view transfer frames (in hex) from PC to Expert and back

	// make "enable transfer frames log" visible when memo log is visible
	CheckBoxEnableLog->Visible = MemoLog->Visible;

	// make "show recent frames" visible when memo log is visible
	ButtonShowLog->Visible = MemoLog->Visible;

}
//---------------------------------------------------------------------------

// PKy 3_2017. Logging is handled in SerialHandler so make like this
#include "DataHandler.h"
#include "SerialHandler.h"

void __fastcall TFormStartMenu::ButtonShowLogClick(TObject *Sender)
{
	// fetch log from devices.
	if (GDeviceManager->GetDeviceCount() == 0)
	{
		MemoLog->Lines->Add("No device connected");
		return;
	}

	TDevice* LDevice = GDeviceManager->GetDevice(0);
	TLogRow* LReadList = LDevice->FDataHandler->GetReadLogStart();
	TLogRow* LWriteList = LDevice->FDataHandler->GetWriteLogStart();

	TStringList* LogItems = new TStringList();
	TLogRow* LCurrentRead = LReadList;
	TLogRow* LCurrentWrite = LWriteList;

	// start timestamp, other times are reduced from here
	__int64 LStartTimestamp = LReadList->FTimestamp;

	// get divider for timestamp checking
	LARGE_INTEGER LInt;
	QueryPerformanceFrequency(&LInt);
	__int64 LPerformanceCounterFrequency = LInt.QuadPart;
	double LPerformanceCounterPerMicroS = LPerformanceCounterFrequency / 1000000.0;
	while (LCurrentRead != NULL || LCurrentWrite != NULL)
	{
		// both lists have items left, pick next message from the old item and advance it in the list
		// OR or read list is the only one having an item
		if ( (LCurrentRead != NULL && LCurrentWrite != NULL &&
			LCurrentRead->FTimestamp < LCurrentWrite->FTimestamp) ||
			LCurrentWrite == NULL)
		{   // view received bytes
			AnsiString LMessage = "EX: ";

			// microseconds
			int LTimeFromLogStart = (int)((LCurrentRead->FTimestamp - LStartTimestamp) / LPerformanceCounterPerMicroS);
			LMessage.cat_printf("(%dus) ", LTimeFromLogStart);

			for (int i = 0; i < LCurrentRead->FByteCount; i++)
			{
				LMessage.cat_printf("%c", (LCurrentRead->GetMessagePtr())[i]);
			}

			LCurrentRead = LCurrentRead->FPtrToNext;
			LogItems->Add(LMessage);
		}    // otherwise spent item from write list
		else
		{   // view sended bytes
			AnsiString LMessage = "PC: ";

			// time is displayed in microseconds
			int LTimeFromLogStart = (int)((LCurrentWrite->FTimestamp - LStartTimestamp) / LPerformanceCounterPerMicroS);
			LMessage.cat_printf("(%dus) ", LTimeFromLogStart);

			for (int i = 0; i < LCurrentWrite->FByteCount; i++)
			{
				LMessage.cat_printf("%c", (LCurrentWrite->GetMessagePtr())[i]);
			}

			LCurrentWrite = LCurrentWrite->FPtrToNext;
			LogItems->Add(LMessage);
		}
	}

	MemoLog->Lines = LogItems;
}
//---------------------------------------------------------------------------

void __fastcall TFormStartMenu::CheckBoxEnableLogClick(TObject *Sender)
{
	// fetch log from devices.
	if (GDeviceManager->GetDeviceCount() == 0)
	{
		MemoLog->Lines->Add("No device connected");
		return;
	}

	TDevice* LDevice = GDeviceManager->GetDevice(0);
	LDevice->FDataHandler->SetLogEnabled(CheckBoxEnableLog->Checked);
	MemoLog->Lines->Add(CheckBoxEnableLog->Checked ? "Logging on" : "Logging off");
}
//---------------------------------------------------------------------------

