//---------------------------------------------------------------------------
// UnitFormStartupSplash.cpp
//
// Noptel/FN Expert (Field), NOS Pro
// C. Noptel Oy, Oulu Finland, 2009-2017
//
// Version: 2.1.6
// Date: 2017-04-04
// Expert FW version: 36.2
// Authors: Noptel/PKy,MHi
//
// Reference: Noptel Expert - Application User's Guide
//            Noptel doc#: N41839 v.2.1
//---------------------------------------------------------------------------


#include <vcl.h>
#pragma hdrstop

#include "UnitFormStartupSplash.h"

// select training dialog
#include "UnitFormStartMenu.h"

// device manager to initialize scan
#include "DeviceManager.h"

// to create shooter forms at startup
#include "UnitFormShooter.h"

#include "ShotData.h"

#include "ShotString.h"

#include "Grading.h"

#include "Globals.h"

// to load name list
#include "UnitFormTeamEdit.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormStartupSplash *FormStartupSplash;
//---------------------------------------------------------------------------
__fastcall TFormStartupSplash::TFormStartupSplash(TComponent* Owner)
	: TForm(Owner)
{
	// load startup splash bitmap, if exists
	// this is loaded here before showing so that it wont be blank for that brief time
	if (FileExists("bitmaps\\start.bmp"))
	{
		try
		{
			ImageSplash->Picture->Bitmap->LoadFromFile("bitmaps\\start.bmp");
			Width = ImageSplash->Picture->Bitmap->Width + 4;
			Height = ImageSplash->Picture->Bitmap->Height + 4;
			// set left position of logo according to start picture width
			ImageApplicationLogo->Left = ImageSplash->Picture->Bitmap->Width - 255;
		}
		catch (Exception& e)
		{
			// not needed
		}
	}
	else // if there was no "Start.bmp", display logo only
	{
		ImageApplicationLogo->Left = 175;
		ImageApplicationLogo->Top = 35;
		Width = 600;
		Bevel->Visible = true;
	}

	// display custom application logo instead of default "Noptel Expert App", if found
	if (FileExists("bitmaps\\logo.bmp"))
	{
		try
		{
			ImageApplicationLogo->Picture->Bitmap->LoadFromFile("bitmaps\\logo.bmp");
		}
		catch (Exception& e)
		{
			// not needed
		}
	}

	// display vendor logo instead of default FN Herstal logo, if found
	if (FileExists("bitmaps\\vendor.bmp"))
	{
		try
		{
			ImageVendorLogo->Picture->Bitmap->LoadFromFile("bitmaps\\vendor.bmp");
		}
		catch (Exception& e)
		{
			// not needed
		}
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormStartupSplash::FormCreate(TObject *Sender)
{
	AnsiString path;
	path = GetCurrentDir();

	// this form is shown in application start (WinMain) before Application->Run()
	// NOSPRO.cpp

	// this both create the device manager and updates port list
	GetDeviceManager()->UpdatePortList();
	// show port listing before actual scan (reported Noptel devices)
	UnicodeString LPorts = WGetTranslation("Ports");
	LPorts += ":\n";

	for (int i = 0; i < GDeviceManager->FPorts->Count; i++)
	{
		//LPorts += "COM" + GDeviceManager->FPorts->Strings[i] ;
	}

	//LabelDevicesInPorts->Caption = LPorts;
	//LabelDevicesInPorts->Update();
	LabelDevicesInPorts->Caption = "";

	// so that no exceptions are raised here if fails to open a port
	TimerStartScan->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TFormStartupSplash::TimerStartScanTimer(TObject *Sender)
{
	// after: so that no exceptions are raised here if fails to open a port
	TimerStartScan->Enabled = false;

	// enable updating status only after scan is started
	TimerUpdateStatus->Enabled = true;

	// initialize, create connect, no download
	GDeviceManager->CheckForDevices(true, true, false);

	// disabled scanning for bluetooth device until further
	//GCheckForBluetooth = false;

	// THIS IS HERE BECAUSE NAME EDITOR IS NOT CREATED WHEN PROCESSING AppInitialize
	// load team&name list file if was previously set
	UnicodeString LFile = GetAppSettingsOptionValueStr(CAppSettingsOptionsTeamNameListFile);
	if (FileExists(LFile) )
	{
		bool LIsOk = FormNameListEditor->FNameHandler.LoadFromFile(LFile);
		if (LIsOk)
		{
			FormNameListEditor->UpdateTeamList();
			FormNameListEditor->UpdateShooterList();
		}
	}

}
//---------------------------------------------------------------------------


void __fastcall TFormStartupSplash::TimerHideSplashTimer(TObject *Sender)
{
	// do not hide splash until scan is completed
	if (GDeviceManager->GetScanningCount() > 0) return;

	TimerHideSplash->Enabled = false;
	Hide();
	FormStartMenu->Show();
}
//---------------------------------------------------------------------------

void __fastcall TFormStartupSplash::ImageSplashClick(TObject *Sender)
{
	TimerHideSplashTimer(NULL);
	TimerHideSplash->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TFormStartupSplash::TimerUpdateStatusTimer(TObject *Sender)
{
    bool LIsGoing = GetDeviceManager()->GetScanningCount() > 0;

	if (!LIsGoing)
	{
		UnicodeString LPorts = WGetTranslation("Found devices: ");

		if (GDeviceManager->GetDeviceCount() > 0)
		{
			// update device icon
			ImageDeviceStatus->Picture = ImageDeviceFound->Picture;

			for (int i = 0; i < GetDeviceManager()->GetDeviceCount(); i++)
			{
				L" ";

				LPorts.cat_printf(L" COM%d", GetDeviceManager()->GetDevice(i)->FCOMPort );
			}
		}
		else
		{
			LPorts = WGetTranslation("No devices found.");
		}

		LabelDevicesInPorts->Caption = LPorts;

		// hide splash after brief and show select dialog
        TimerUpdateStatus->Enabled = false;
		TimerHideSplash->Enabled = true;
	}
	else
	{
		int L1 = GDeviceManager->GetScanningCount();
		int L2 = GDeviceManager->GetFoundCount();
		UnicodeString LPorts;
		LPorts.printf(WGetTranslation("Scanning %d").c_str(), L1);
		LPorts += ": ";
		UnicodeString LScanning, LFound;

		for (int i = 0; i < GDeviceManager->GetDeviceCount(); i++)
		{

			if (GDeviceManager->GetDevice(i)->FAsyncInfo.FScanGoing &&
				!GDeviceManager->GetDevice(i)->FAsyncInfo.FScanDone)
			{
				if (GDeviceManager->GetDevice(i)->FCOMPort > 0)
					LScanning.cat_printf(L" COM%d", GDeviceManager->GetDevice(i)->FCOMPort);
			}
			else if (GDeviceManager->GetDevice(i)->FInitDeviceReady)
			{
				LFound.cat_printf(L" COM%d", GDeviceManager->GetDevice(i)->FCOMPort);
            }
		}

		LPorts += LScanning;
		//LPorts += L"\n";
		//LPorts.cat_printf(WGetTranslation("Found %d").c_str(), L2);
		//LPorts += ": ";
		LPorts += LFound;


		LabelDevicesInPorts->Caption = LPorts;
    }
}
//---------------------------------------------------------------------------



